#ifndef ITEMIZEENVIRONMENT_H
#define ITEMIZEENVIRONMENT_H


#include "environment.h"


/** This class represents an <ITEMIZE>-environment.
    This environment contains at least one item (ItemizeElement) stored in an
    Element-vector (derived from Environment).
    Each item may have an embedded <ITEMIZE>-subenvironment.
*/
class ItemizeEnvironment : public Environment {
  public:
    /** Generates a new <ITEMIZE>-environment with the given parameters.
	@param write_one_by_one Should the items be written one-by-one ?
	@param level The level of the itemization (for nested itemizations)
	@param with_left_margin Indicates, if the HTML-output should have a
	       margin on the left side
    */
    ItemizeEnvironment(int write_one_by_one,unsigned int level=0,
		       int with_left_margin=1);

    /// Destructor
    ~ItemizeEnvironment();


    /** This method generates an <ITEMIZE>-environment out of the given
	input filestream.
	The input filestream will be read until an </ITEMIZE>-Tag is reached.
	@param input The input filestream, from where the items are extracted
    */
    void read(ifstream *input);

    /** This method writes the given number of items in the item-list to the
	given output filestream.
	If number_of_items is -1, the helper-method writeAllItems() is called.
        Else, the helper-method writeItemize() is called.
	@param output The output filestream
	@param number_of_items The number of items that should be written.
    */
    void write(ofstream *output,int number_of_items);

    /** This method indicates, if the items should be written one-by-one,
	or all items should be written at once.
	@return 1, if the items should be written one-by-one, else 0.
    */
    int shouldBeWrittenOneByOne();

    /** This method returns the number of items in the environment.
	@return The number of items
    */
    int getNumberOfElements();

  private:
    /** This helper-method writes an <UL>-Header with the apropriate
	parameters to the given output filestream.
	@param output The output filestream
     */
    void writeHeader(ofstream *output);

    /** This helper-method writes the given number of items to the given
        output filestream. The first number_of_items-1 items will be written
	WEAK and the last item will be written NORMAL.
	@param output The output filestream
	@param number_of_items The number of items that should be written.
    */
    void writeItemize(ofstream *output,int number_of_items);

    /** This helper-method writes all items to the given output filestream
	with the given style.
	@param output The output filestream
	@param style How the items should be printed (NORMAL or WEAK)
     */
    void writeAllItems(ofstream *output,int style);

    /** This helper-method returns the number of characters of the longest
	item.
	@return The length of the longest item
    */
    unsigned int getMaxLength();


  private:
    /// 1, if the items should be written one-by-one, else 0
    int write_one_by_one_;

    /// Indicates the level of the itemization.
    unsigned int level_;

    /// Indicates, if the HTML-output should have a margin on the left side.
    int with_left_margin_;
    
    /// The number of items in this itemization.
    unsigned int number_of_items_;
};

#endif //ITEMIZEENVIRONMENT_H
