#include <dirent.h>
#include <fstream.h>

#include "main.h"
#include "parameters.h"

#include <magick/magick.h>


Parameters::Parameters()
{
    input_file_ = NULL;
    path_to_themes_ = NULL;
    theme_ = NULL;
    target_directory_ = NULL;
    path_to_theme_ = NULL;
    resolution_ = DEFAULT_RESOLUTION;

    enable_background_graphic_ = 1;
    enable_header_graphic_ = 1;
    enable_footer_graphic_ = 1;

    cout << "Initializing the parameters with default-values ..." << endl;

    setPathToThemes(DEFAULT_PATH_TO_THEMES);
    setTheme(DEFAULT_THEME);
    setTargetDirectory(DEFAULT_TARGET_DIRECTORY);
}

Parameters::~Parameters()
{
    if (input_file_)  delete [] input_file_;
    if (path_to_themes_)  delete [] path_to_themes_;
    if (theme_)  delete [] theme_;
    if (target_directory_)  delete [] target_directory_;
    if (path_to_theme_) {
        path_to_theme_->freeze(0);
	delete path_to_theme_;
    }
}


int Parameters::readConfigFile(const char *filename)
{
    ifstream input(filename);
    if (!input) {
        cerr << endl << "The config-file \"" << filename
	     << "\" doesn't exist." << endl;
	return -1;
    }

    cout << endl << "Reading the config-file \"" << filename
	 << "\" ..." << endl;

    unsigned int current_line_number = 0;
    unsigned int number_of_errors = 0;

    char option[BUFFER_SIZE];
    char equals;
    char value[BUFFER_SIZE];

    while (!input.eof()) {
        input.getline(Buffer_,BUFFER_SIZE);
	current_line_number++;

	if ((strlen(Buffer_) > 0) && (Buffer_[0] != '#')) {
	    istrstream line(Buffer_);
	    line >> option >> equals >> value;  // "option = value"

	    // Check the line for errors
	    if (strlen(option) == 0) {
	        cerr << "Error in \"" << filename << "\" at line "
		     << current_line_number << " :" << endl
		     << "    An option is missing there." << endl;
		number_of_errors++;

	    } else if (strlen(value) == 0) {
	        cerr << "Error in \"" << filename << "\" at line "
		     << current_line_number << " :" << endl
		     << "    The value of the option \"" << option
		     << "\" is missing." << endl;
		number_of_errors++;

	    } else if (equals != '=') {
	        cerr << "Error in \"" << filename << "\" at line "
		     << current_line_number << " :" << endl
		     << "    A '=' is missing there." << endl;
		number_of_errors++;
	    }

	    // Parse the line of the config-file
	    if (!strcmp(option,OPTION_PATH_TO_THEMES)) {
	        setPathToThemes(value);

	    } else if (!strcmp(option,OPTION_THEME)) {
	        setTheme(value);

	    } else if (!strcmp(option,OPTION_TARGET_DIRECTORY)) {
	        setTargetDirectory(value);

	    } else if (!strcmp(option,OPTION_RESOLUTION)) {
		if (!strcmp(value,"800x600")) {
		    resolution_ = 800;
		} else if (!strcmp(value,"1024x768")) {
		    resolution_ = 1024;
		} else {
		    cerr << "Error in \"" << filename << "\" at line "
			 << current_line_number << " :" << endl
			 << "    The resolution must be 800x600 or 1024x768."
			 << endl;
		    number_of_errors++;
		}

	    } else {
	        cerr << "Error in \"" << filename << "\" at line "
		     << current_line_number << " :" << endl
		     << "    \"" << option << "\" is not a valid option."
		     << endl;
		number_of_errors++;
	    }
	}
    }

    if (number_of_errors > 0) {
        cerr << "The " << (number_of_errors == 1 ? "error" : "errors")
	     << " in the config-file \"" << filename
	     << "\" will be ignored." << endl
	     << "Please fix " << (number_of_errors == 1 ? "it" : "them")
	     << "." << endl;
    }

    return number_of_errors;
}

void Parameters::setPathToThemes(const char *path_to_themes)
{
    if (path_to_themes) {
        DIR *directory = opendir(path_to_themes);
	if (directory) {
	    closedir(directory);
	    if (path_to_themes_)  delete [] path_to_themes_;
	    path_to_themes_ = new char[strlen(path_to_themes)+2];
	    strcpy(path_to_themes_,path_to_themes);
	    if (path_to_themes_[strlen(path_to_themes_)-1] != '/') {
	        strcat(path_to_themes_,"/");
	    }
	} else {
	    cerr << "The theme-path \"" << path_to_themes
		 << "\" doesn't exist." << endl;
	    if (path_to_themes_) {
	        cerr << "I will keep \"" << path_to_themes_
		     << "\" as theme-path." << endl;
	    } else {
	        cerr << "The theme-path remains uninitialized !" << endl;
	    }
	}
    }
}

void Parameters::setTheme(const char *theme)
{
    if (theme_)  delete [] theme_;
    if (theme) {
        theme_ = new char[strlen(theme)+1];
	strcpy(theme_,theme);
    } else {
        theme_ = NULL;
    }
}

void Parameters::setTargetDirectory(const char *target_directory)
{
    if (target_directory) {
        DIR *directory = opendir(target_directory);
	if (directory) {
	    closedir(directory);
	    if (target_directory_)  delete [] target_directory_;
	    target_directory_ = new char[strlen(target_directory)+2];
	    strcpy(target_directory_,target_directory);
	    if (target_directory_[strlen(target_directory_)-1] != '/') {
	        strcat(target_directory_,"/");
	    }
	} else {
	    cerr << "The target-directory \"" << target_directory
		 << "\" doesn't exist." << endl
		 << "I will keep \"" << target_directory_
		 << "\" as target-directory." << endl;
	}
    }
}


int Parameters::initPathToTheme()
{
    int return_code = 0;

    if (path_to_theme_) {
        path_to_theme_->freeze(0);
	delete path_to_theme_;
    }
    path_to_theme_ = new ostrstream();
    *path_to_theme_ << path_to_themes_ << theme_ << '/' << ends;

    // Check, if there is a file "config" in the theme-directory
    ostrstream config;
    config << path_to_theme_->str() << "config" << ends;
    ifstream theme_config_file(config.str());
    if (!theme_config_file) {
        return_code = -1;
	cerr << "The theme \"" << theme_ << "\" doesn't exist in \""
	     << path_to_themes_ << "\"." << endl;
    }
    config.freeze(0);

    return return_code;
}

void Parameters::initImageSizes()
{
    prev_ = NULL;
    next_ = NULL;
    slide_ = NULL;
    header_ = NULL;
    footer_ = NULL;

    Image *image;
    ImageInfo ii;
    GetImageInfo(&ii);

    // Initialize the size of the previous-button
    sprintf(ii.filename,"%s%s",path_to_theme_->str(),PREV);
    image = PingImage(&ii);
    if (image) {
        prev_ = new ImageSize;
	prev_->width_ = image->columns;
	prev_->height_ = image->rows;
	DestroyImage(image);
    } else {
        cerr << "Warning : The image \"" << PREV << "\" doesn't exist."
	     << endl;
    }

    // Initialize the size of the next-button
    sprintf(ii.filename,"%s%s",path_to_theme_->str(),NEXT);
    image = PingImage(&ii);
    if (image) {
        next_ = new ImageSize;
	next_->width_ = image->columns;
	next_->height_ = image->rows;
	DestroyImage(image);
    } else {
        cerr << "Warning : The image \"" << NEXT << "\" doesn't exist."
	     << endl;
    }

    // Initialize the size of the background-graphic for the slide
    if (enable_background_graphic_ == 1) {
	sprintf(ii.filename,"%s%s%s",path_to_theme_->str(),BG_SLIDE,JPG_EXT);
	image = PingImage(&ii);
	if (image) {
	    slide_ = new ImageSize;
	    slide_->width_ = image->columns;
	    slide_->height_ = image->rows;
	    DestroyImage(image);
	} else {
	    cerr << "Warning : The image \"" << BG_SLIDE << JPG_EXT
		 << "\" doesn't exist." << endl << "          "
		 << "The background-graphic for the slides will be disabled."
		 << endl;
	    enable_background_graphic_ = 0;
	}
    }

    // Initialize the size of the backgound-graphic for the header
    if (enable_header_graphic_ == 1) {
	sprintf(ii.filename,"%s%s_%d%s",path_to_theme_->str(),
		BG_HEADER,resolution_,GIF_EXT);
	image = PingImage(&ii);
	if (image) {
	    header_ = new ImageSize;
	    header_->width_ = image->columns;
	    header_->height_ = image->rows;
	    DestroyImage(image);
	} else {
	    cerr << "Warning : The image \"" << BG_HEADER << "_" << resolution_
		 << GIF_EXT << "\" doesn't exist." << endl << "          "
		 << "The background-graphic for the header will be disabled."
		 << endl;
	    enable_header_graphic_ = 0;
	}
    }

    // Initialize the size of the backgound-graphic for the footer
    if (enable_footer_graphic_ == 1) {
	sprintf(ii.filename,"%s%s_%d%s",path_to_theme_->str(),
		BG_FOOTER,resolution_,GIF_EXT);
	image = PingImage(&ii);
	if (image) {
	    footer_ = new ImageSize;
	    footer_->width_ = image->columns;
	    footer_->height_ = image->rows;
	    DestroyImage(image);
	} else {
	    cerr << "Warning : The image \"" << BG_FOOTER << "_" << resolution_
		 << GIF_EXT << "\" doesn't exist." << endl << "          "
		 << "The background-graphic for the footer will be disabled."
		 << endl;
	    enable_footer_graphic_ = 0;
	}
    }
}


char *Parameters::getPathToTheme()
{
    return path_to_theme_->str();
}
