#ifndef PARAMETERS_H
#define PARAMETERS_H


#include <config.h>

#include <strstream.h>

#include "imagesize.h"


#define DEFAULT_THEME "standard_blue"
#define DEFAULT_TARGET_DIRECTORY "./" 
#define DEFAULT_RESOLUTION 800


/** This class holds all parameters for the presentation.
    This parameters will be initialized with hard-coded default-values.
    The values can be overridden by a global config-file, a
    user config-file (located in the user's home-directory) and by
    commandline-options.
 */
class Parameters {
  public:
    /// Default constructor which initializes the parameters to default-values.
    Parameters();

    /// Destructor
    ~Parameters();

    /** This method reads the given config-file and updates the parameters.
	@param filename The name of the config-file
	@return -1 if the file doesn't exist, 0 if everything is OK, else
	        the number of errors in the config-file
    */
    int readConfigFile(const char *filename);

    /** This method will set the path to the directory, which contains all
	themes.
	A trailing '/' will be added if the parameter doesn't have one.
	@param path_to_themes The new path to the themes
    */
    void setPathToThemes(const char *path_to_themes);

    /** This method will set the theme that should be used for the
	presentation.
	@param theme The new theme
    */
    void setTheme(const char *theme);

    /** This method will set the target-directory, where all presentation-files
	will be written.
	A trailing '/' will be added if the parameter doesn't have one.
	@param target_directory The new target-directory
    */
    void setTargetDirectory(const char *target_directory);

    /** This method initializes the path to the theme.
	This method must be called after the config-file and the
	commandline-parameters are evaluated.
	@return 0, if the theme exists, else -1
    */
    int initPathToTheme();

    /** This method initializes the sizes of the background-images.
	This method must be called after the path to the theme was initialized.
    */
    void initImageSizes();

    /** This method returns the path to the selected theme.
	@return The path to the selected theme
    */
    char *getPathToTheme();


  public:
    /// The name of the inputfile
    char *input_file_;

    /// The path to the directory which contains all themes
    char *path_to_themes_;

    /// The theme that will be used for the presentation
    char *theme_;

    /// The target-directory where the presentation-files will be written
    char *target_directory_;

    /// The screen-resolution
    unsigned int resolution_;

  private:
    /// The path to the selected theme
    ostrstream *path_to_theme_;


  public:
    /// 1, if the background-graphic for the slide should be enabled, else 0
    int enable_background_graphic_;

    /// 1, if the background-graphic for the header should be enabled, else 0
    int enable_header_graphic_;

    /// 1, if the background-graphic for the footer should be enabled, else 0
    int enable_footer_graphic_;


    /// The dimensions of the previous-button
    ImageSize *prev_;

    /// The dimensions of the next-button
    ImageSize *next_;

    /// The dimensions of the background-graphic for the slide
    ImageSize *slide_;

    /// The dimensions of the background-graphic for the header
    ImageSize *header_;

    /// The dimensions of the background-graphic for the footer
    ImageSize *footer_;
};

#endif //PARAMETERS_H
