#include <unistd.h>
#include <fstream.h>
#include <iomanip.h>

#include "main.h"
#include "parameters.h"
#include "presentation.h"


Presentation::Presentation()
{
    slides_.clear();
}


Presentation::~Presentation()
{
    for (unsigned int i=0;i<slides_.size();i++) {
        delete slides_[i];
    }
    slides_.clear();
}


int Presentation::read()
{
    ifstream input(Parameters_->input_file_);
    if (!input) {
        cerr << "The inputfile \"" << Parameters_->input_file_
	     << "\" doesn't exist." << endl;
	return -1;
    }

    // Read the slides from the inputfile
    Slide *slide;
    unsigned int number_of_slides = 0;
    while (!input.eof()) {
        slide = new Slide(++number_of_slides);
	slide->read(&input);
	slides_.push_back(slide);
    }

    if (Number_Of_Errors_ > 0) {
        cerr << "There " << (Number_Of_Errors_ == 1 ? "is" : "are")
	     << " " << Number_Of_Errors_ << " "
	     << (Number_Of_Errors_ == 1 ? "error" : "errors")
	     << " in the inputfile." << endl
	     << "No slides will be created." << endl;
    }

    return Number_Of_Errors_;
}


void Presentation::write()
{
    // Copy the next-button to the target-directory
    ostrstream next_src,next_dst;
    next_src << Parameters_->getPathToTheme() << NEXT << ends;
    next_dst << Parameters_->target_directory_ << NEXT << ends;
    fcopy(next_src.str(),next_dst.str());
    next_src.freeze(0);
    next_dst.freeze(0);

    // Copy the prev-button to the target-directory
    ostrstream prev_src,prev_dst;
    prev_src << Parameters_->getPathToTheme() << PREV << ends;
    prev_dst << Parameters_->target_directory_ << PREV << ends;
    fcopy(prev_src.str(),prev_dst.str());
    prev_src.freeze(0); prev_dst.freeze(0);

    // Copy the stylesheet to the target-directory
    ostrstream styles_src,styles_dst;
    styles_src << Parameters_->getPathToTheme() << STYLES << "_"
	       << Parameters_->resolution_ << STYLES_EXT << ends;
    styles_dst << Parameters_->target_directory_ << STYLES << STYLES_EXT
	       << ends;
    fcopy(styles_src.str(),styles_dst.str());
    styles_src.freeze(0); styles_dst.freeze(0);

    // Copy the background-image for the slide to the target-directory,
    // or remove it from there.
    ostrstream bg_slide_src,bg_slide_dst;
    bg_slide_src << Parameters_->getPathToTheme() << BG_SLIDE << JPG_EXT
		 << ends;
    bg_slide_dst << Parameters_->target_directory_ << BG_SLIDE << JPG_EXT
		 << ends;
    if (Parameters_->enable_background_graphic_ == 1) {
	fcopy(bg_slide_src.str(),bg_slide_dst.str());
    } else {
        unlink(bg_slide_dst.str());
    }
    bg_slide_src.freeze(0); bg_slide_dst.freeze(0);

    // Copy the background-graphic for the header to the target-directory,
    // or remove it from there.
    ostrstream bg_header_src,bg_header_dst;
    bg_header_src << Parameters_->getPathToTheme() << BG_HEADER << "_"
		  << Parameters_->resolution_ << GIF_EXT << ends;
    bg_header_dst << Parameters_->target_directory_ << BG_HEADER << GIF_EXT
		  << ends;
    if (Parameters_->enable_header_graphic_ == 1) {
	fcopy(bg_header_src.str(),bg_header_dst.str());
    } else {
        unlink(bg_header_dst.str());
    }
    bg_header_src.freeze(0); bg_header_dst.freeze(0);

    // Copy the background-graphic for the footer to the target-directory,
    // or remove it from there.
    ostrstream bg_footer_src,bg_footer_dst;
    bg_footer_src << Parameters_->getPathToTheme() << BG_FOOTER << "_"
		  << Parameters_->resolution_ << GIF_EXT << ends;
    bg_footer_dst << Parameters_->target_directory_ << BG_FOOTER << GIF_EXT
		  << ends;
    if (Parameters_->enable_footer_graphic_ == 1) {
	fcopy(bg_footer_src.str(),bg_footer_dst.str());
    } else {
        unlink(bg_footer_dst.str());
    }
    bg_footer_src.freeze(0); bg_footer_dst.freeze(0);


    // Write the index-page
    cout << "Creating the index-page ... " << flush;
    writeIndexPage();
    cout << "finished." << endl;


    // Write the slides
    cout << "Creating " << slides_.size() << " slides ... " << flush;
    for (unsigned int i=0;i<slides_.size();i++) {
        slides_[i]->write();
    }
    cout << "finished." << endl;
}


Slide *Presentation::getPrevSlide(unsigned int slide_number)
{
    return ((slide_number > 1) && (slide_number <= slides_.size()) ?
	    slides_[slide_number-2] : NULL);
}

Slide *Presentation::getNextSlide(unsigned int slide_number)
{
    return ((slide_number >= 1) && (slide_number < slides_.size()) ?
	    slides_[slide_number] : NULL);
}


void Presentation::writeIndexPage()
{
    ostrstream filename;
    filename << Parameters_->target_directory_ << "index.html" << ends;

    ofstream output(filename.str());

    output << "<HTML>" << endl
	   << "<HEAD>" << endl
	   << "<TITLE>Index of the presentation</TITLE>" << endl
	   << "</HEAD>" << endl
	   << "<BODY>" << endl
	   << "<TABLE>" << endl;
      
    for (unsigned int i=0;i<slides_.size();i++) {
        ostrstream filename_of_slide;
	filename_of_slide << setfill('0') << setw(2)
			  << slides_[i]->sequence_number_;
	if (slides_[i]->number_of_subslides_ > 1) {
	    filename_of_slide << "_1";
	}
	filename_of_slide << ".html" << ends;

	output << "<TR><TD>" << endl
	       << "<A href=\"" << filename_of_slide.str() << "\">"
	       << slides_[i]->getTitle() << "</A>" << endl
	       << "</TD></TR>" << endl;

        filename_of_slide.freeze(0);
    }

    output << "</TABLE>" << endl
	   << "</BODY>" << endl
	   << "</HTML>" << endl;

    filename.freeze(0);
}
