#include <stdlib.h>
#include <iomanip.h>

#include "main.h"
#include "slide.h"
#include "rawenvironment.h"
#include "itemizeenvironment.h"
#include "imageenvironment.h"
#include "hboxenvironment.h"
#include "gridenvironment.h"
#include "parameters.h"
#include "presentation.h"


Slide::Slide(unsigned int sequence_number)
{
    sequence_number_ = sequence_number;
    number_of_subslides_ = -1;
    environment_ = NULL;
}

Slide::~Slide()
{
    if (environment_)  delete environment_;
    title_.freeze(0);
}


void Slide::read(ifstream *input)
{
    unsigned int line_of_slide = 1;

    input->getline(Buffer_,BUFFER_SIZE);
    Current_Line_Number_++;

    while (strlen(Buffer_) > 0) {
        if (Buffer_[0] != '#') {
	    if (line_of_slide == 1) {
	        title_ << Buffer_ << ends;

	    } else if (strstr(Buffer_,"<GRID>")) {
	        environment_ = new GridEnvironment(0);
		environment_->read(input);

	    } else if (strstr(Buffer_,"<GRID STEP>")) {
	        environment_ = new GridEnvironment(1);
		environment_->read(input);

	    } else if (strstr(Buffer_,"</GRID>")) {
	        cerr << "Line " << Current_Line_Number_
		     << ": There's a <GRID> tag missing." << endl;
		Number_Of_Errors_++;

	    } else if (strstr(Buffer_,"<HBOX>")) {
	        environment_ = new HBoxEnvironment();
		environment_->read(input);

	    } else if (strstr(Buffer_,"</HBOX>")) {
	        cerr << "Line " << Current_Line_Number_
		     << ": There's a <HBOX> tag missing." << endl;
		Number_Of_Errors_++;

	    } else if (strstr(Buffer_,"<ITEMIZE>")) {
	        environment_ = new ItemizeEnvironment(0);
		environment_->read(input);

	    } else if (strstr(Buffer_,"<ITEMIZE STEP>")) {
	        environment_ = new ItemizeEnvironment(1);
		environment_->read(input);

	    } else if (strstr(Buffer_,"</ITEMIZE>")) {
	        cerr << "Line " << Current_Line_Number_
		     << ": There's an <ITEMIZE> tag missing." << endl;
		Number_Of_Errors_++;

	    } else if (strstr(Buffer_,"<IMG>")) {
	        environment_ = new ImageEnvironment();
		environment_->read(input);

	    } else if (strstr(Buffer_,"</IMG>")) {
	        cerr << "Line " << Current_Line_Number_
		     << ": There's an <IMG> tag missing." << endl;
		Number_Of_Errors_++;

	    } else if (strstr(Buffer_,"<RAW>")) {
	        environment_ = new RawEnvironment();
		environment_->read(input);

	    } else if (strstr(Buffer_,"</RAW>")) {
	        cerr << "Line " << Current_Line_Number_
		     << ": There's a <RAW> tag missing." << endl;
	    }

	    line_of_slide++;
	}
	input->getline(Buffer_,BUFFER_SIZE);
	Current_Line_Number_++;
    }

    if (line_of_slide == 1) {
        cerr << "Line " << Current_Line_Number_
	     << ": There should be only one empty line." << endl;
	Number_Of_Errors_++;
    }

    if (environment_ && environment_->shouldBeWrittenOneByOne()) {
        number_of_subslides_ = environment_->getNumberOfElements();
    }
}


void Slide::write()
{
    if (number_of_subslides_ == -1) {  // No subslides
        writeSubSlide(-1);

    } else {  // There are "number_of_subslides_" subslides to write
        for (int i=1;i<=number_of_subslides_;i++) {
	    writeSubSlide(i);
	}
    }
}

char *Slide::getTitle()
{
    return title_.str();
}

void Slide::writeSubSlide(int sub_slide)
{
    // Determine the filename of this (sub)slide
    ostrstream filename;

    filename << Parameters_->target_directory_ << setfill('0') << setw(2)
	     << sequence_number_;
    if (sub_slide > 0) {
        filename << "_" << sub_slide;
    }
    filename << ".html" << ends;


    // Determine the filename of the previous slide
    ostrstream filename_of_prev_slide;
    Slide *prev_slide = Presentation_->getPrevSlide(sequence_number_);
    if (prev_slide || (sub_slide > 1)) {
        if (sub_slide > 1) {
	    filename_of_prev_slide << setfill('0') << setw(2)
				   << sequence_number_ << "_" << sub_slide-1
				   << ".html" << ends;

	} else if (prev_slide->number_of_subslides_ == -1) {
	    filename_of_prev_slide << setfill('0') << setw(2)
				   << prev_slide->sequence_number_ << ".html"
				   << ends;

	} else {
	    filename_of_prev_slide << setfill('0') << setw(2)
				   << prev_slide->sequence_number_ << "_"
				   << prev_slide->number_of_subslides_
				   << ".html" << ends;
	}
    }


    // Determine the filename of the next slide
    ostrstream filename_of_next_slide;
    Slide *next_slide = Presentation_->getNextSlide(sequence_number_);
    if (next_slide || ((sub_slide>0) && (sub_slide<number_of_subslides_))) {
        if ((sub_slide > 0) && (sub_slide < number_of_subslides_)) {
	    filename_of_next_slide << setfill('0') << setw(2)
				   << sequence_number_ << "_" << sub_slide+1
				   << ".html" << ends;

	} else if (next_slide->number_of_subslides_ == -1) {
	    filename_of_next_slide << setfill('0') << setw(2)
				   << next_slide->sequence_number_ << ".html"
				   << ends;

	} else {
	    filename_of_next_slide << setfill('0') << setw(2)
				   << next_slide->sequence_number_ << "_1"
				   << ".html" << ends;
	}
    }

    // Write the slide to the HTML-file
    ofstream *output = new ofstream(filename.str());
    writeHTMLHeader(output,filename_of_prev_slide.str(),
		    filename_of_next_slide.str());
    writeHeader(output);
    if (environment_) {
        environment_->write(output,sub_slide);
    }
    writeFooter(output,
		filename_of_prev_slide.str(),filename_of_next_slide.str());

    filename.freeze(0);
    filename_of_prev_slide.freeze(0);
    filename_of_next_slide.freeze(0);
    delete output;
}


void Slide::writeHTMLHeader(ofstream *output,char *prev_slide,char *next_slide)
{
    *output << "<HTML>" << endl
	    << "<HEAD>" << endl
	    << "<TITLE>" << title_.str() << "</TITLE>" << endl;

    *output << "<LINK href=\"index.html\" rel=\"contents\" "
	    << "title=\"Contents\">" << endl;

    if (prev_slide) {
        *output << "<LINK href=\"" << prev_slide
		<< "\" rel=\"previous\" title=\"Previous slide\">" << endl;
    }

    if (next_slide) {
	*output << "<LINK href=\"" << next_slide
		<< "\" rel=\"next\" title=\"Next slide\">" << endl;
    }

    *output << "<LINK href=\"styles.css\" rel=\"stylesheet\" "
	    << "type=\"text/css\">" << endl
	    << "</HEAD>" << endl
	    << "<BODY>" << endl;
}

void Slide::writeHeader(ofstream *output)
{
    *output << "<DIV class=\"header\"><TABLE align=\"center\" border=\"0\"";
    if (Parameters_->enable_header_graphic_ == 1) {
        *output << " width=\"" << Parameters_->header_->width_
		<< "\" height=\"" << Parameters_->header_->height_
		<< "\">" << endl;
    } else {
        *output << " width=\"100%\">" << endl;
    }
    *output << "<TR><TD class=\"header\">" << endl
	    << "<H1>" << title_.str() << "</H1>" << endl
	    << "</TD></TR>" << endl
	    << "</TABLE></DIV>" << endl;
}

void Slide::writeFooter(ofstream *output,char *prev_slide,char *next_slide)
{
    *output << "<DIV class=\"footer\"><TABLE border=\"0\" width=\"100%\"><TR>"
	    << endl;


    // previous button
    *output << "<TD align=\"left\" width=\"";
    if (Parameters_->prev_) {
        *output << Parameters_->prev_->width_;
    } else {
        *output << "50%";
    }
    *output << "\">";

    if (prev_slide) {
        *output << "<A href=\"" << prev_slide
		<< "\"><IMG src=\"prev.gif\" border=\"0\"";
	if (Parameters_->prev_) {
	    *output << " width=\"" << Parameters_->prev_->width_
		    << "\" height=\"" << Parameters_->prev_->height_ << "\"";
	}
	*output << "></A>";
    }
    *output << "</TD>" << endl;


    // footer graphic
    if (Parameters_->enable_footer_graphic_ == 1) {
        *output << "<TD align=\"center\"><IMG src=\"bg_footer.gif\""
		<< " height=\"" << Parameters_->footer_->height_
		<< "\" width=\"" << Parameters_->footer_->width_
		<< "\"></TD>" << endl;
    }


    // next button
    *output << "<TD align=\"right\" width=\"";
    if (Parameters_->next_) {
        *output << Parameters_->next_->width_;
    } else {
        *output << "50%";
    }
    *output << "\">";

    if (next_slide) {
        *output << "<A href=\"" << next_slide
		<< "\"><IMG src=\"next.gif\" border=\"0\"";
	if (Parameters_->next_) {
	    *output << " width=\"" << Parameters_->next_->width_
		    << "\" height=\"" << Parameters_->next_->height_ << "\"";
	}
	*output << "></A>";
    }
    *output << "</TD>" << endl;


    *output << "</TR></TABLE></DIV>" << endl
	    << "</BODY>" << endl
	    << "</HTML>" << endl;
}
